package eu.eudat.service.fielddatahelper;

import eu.eudat.authorization.AuthorizationFlags;
import eu.eudat.commons.types.descriptiontemplate.fielddata.ExternalSelectDataEntity;
import eu.eudat.commons.types.descriptiontemplate.importexport.fielddata.*;
import eu.eudat.convention.ConventionService;
import eu.eudat.model.builder.descriptiontemplatedefinition.fielddata.ExternalSelectDataBuilder;
import eu.eudat.model.descriptiontemplatedefinition.fielddata.ExternalSelectData;
import eu.eudat.model.persist.descriptiontemplatedefinition.fielddata.*;
import gr.cite.tools.data.builder.BuilderFactory;
import gr.cite.tools.fieldset.FieldSet;
import gr.cite.tools.validation.Validator;
import gr.cite.tools.validation.ValidatorFactory;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

@Component
public class ExternalSelectFieldDataHelperService extends BaseFieldDataHelperService<ExternalSelectData, ExternalSelectDataPersist, ExternalSelectDataEntity, ExternalSelectImportExport> {
	private final ConventionService conventionService;
	private final BuilderFactory builderFactory;
	private final ValidatorFactory validatorFactory;

	public ExternalSelectFieldDataHelperService(ConventionService conventionService, BuilderFactory builderFactory, ValidatorFactory validatorFactory) {
		this.conventionService = conventionService;
		this.builderFactory = builderFactory;
		this.validatorFactory = validatorFactory;
	}


	@Override
	public ExternalSelectDataEntity newDataInstanceInternal() {
		return new ExternalSelectDataEntity();
	}

	@Override
	public ExternalSelectData newModelInstanceInternal() {
		return new ExternalSelectData();
	}

	@Override
	public ExternalSelectDataPersist newPersistModelInstanceInternal() {
		return new ExternalSelectDataPersist();
	}

	@Override
	protected ExternalSelectImportExport newImportExportInstanceInternal() {
		return new ExternalSelectImportExport();
	}

	@Override
	public Class<?> getDataClass() {
		return ExternalSelectDataEntity.class;
	}
	@Override
	public Class<?> getModelClass() {
		return ExternalSelectData.class;
	}
	@Override
	public Class<?> getPersistModelClass() {
		return ExternalSelectDataPersist.class;
	}

	@Override
	public Class<?> getImportExportClass() {
		return ExternalSelectImportExport.class;
	}

	@Override
	public Validator getPersistModelValidator() {
		return this.validatorFactory.validator(ExternalSelectDataPersist.AutoCompleteDataPersistValidator.class);
	}

	@Override
	public List<ExternalSelectData> buildInternal(FieldSet fieldSet, List<ExternalSelectDataEntity> data, EnumSet<AuthorizationFlags> authorizationFlags){
		return this.builderFactory.builder(ExternalSelectDataBuilder.class).authorize(authorizationFlags).build(fieldSet, data);
	}

	@Override
	protected ExternalSelectDataEntity applyPersistInternal(ExternalSelectDataPersist persist, ExternalSelectDataEntity data) {
		data.setMultipleSelect(persist.getMultipleSelect());
		if (!this.conventionService.isListNullOrEmpty(persist.getSources())){
			data.setSources(new ArrayList<>());
			for (ExternalSelectDataPersist.ExternalSelectSourcePersist sourcePersist: persist.getSources()) {
				data.getSources().add(this.buildAutoCompleteSingleData(sourcePersist));
			}
		}
		return data;
	}

	private @NotNull ExternalSelectDataEntity.ExternalSelectSourceEntity buildAutoCompleteSingleData(ExternalSelectDataPersist.ExternalSelectSourcePersist persist){
		ExternalSelectDataEntity.ExternalSelectSourceEntity data = new ExternalSelectDataEntity.ExternalSelectSourceEntity();
		if (persist == null) return data;

		data.setUrl(persist.getUrl());
		data.setOptionsRoot(persist.getOptionsRoot());
		data.setHasAuth(persist.getHasAuth());
		if(persist.getSourceBinding() != null) data.setSourceBinding(this.buildOption(persist.getSourceBinding()));
		if(persist.getAuth() != null) data.setAuth(this.buildAuthAutoCompleteData(persist.getAuth()));
		data.setMethod(persist.getMethod());
		
		return data;
	}

	private @NotNull ExternalSelectDataEntity.ExternalSelectSourceBindingEntity buildOption(ExternalSelectDataPersist.ExternalSelectSourceBindingPersist persist){
		ExternalSelectDataEntity.ExternalSelectSourceBindingEntity data = new ExternalSelectDataEntity.ExternalSelectSourceBindingEntity();
		if (persist == null) return data;

		data.setLabel(persist.getLabel());
		data.setValue(persist.getValue());
		data.setSource(persist.getSource());

		return data;
	}

	private @NotNull ExternalSelectDataEntity.ExternalSelectAuthDataEntity buildAuthAutoCompleteData(ExternalSelectDataPersist.ExternalSelectAuthDataPersist persist){
		ExternalSelectDataEntity.ExternalSelectAuthDataEntity data = new ExternalSelectDataEntity.ExternalSelectAuthDataEntity();
		if (persist == null) return data;

		data.setBody(persist.getBody());
		data.setUrl(persist.getUrl());
		data.setMethod(persist.getMethod());
		data.setPath(persist.getPath());
		data.setType(persist.getType());

		return data;
	}

	@Override
	protected ExternalSelectDataPersist importExportMapDataToPersistInternal(ExternalSelectImportExport data, ExternalSelectDataPersist persist) {
		persist.setMultipleSelect(data.getMultipleSelect());
		if (!this.conventionService.isListNullOrEmpty(data.getSources())){
			persist.setSources(new ArrayList<>());
			for (ExternalSelectImportExport.ExternalSelectSourceImportExport autoCompleteSingleData: data.getSources()) {
				persist.getSources().add(this.buildAutoCompleteSingleData(autoCompleteSingleData));
			}
		}
		return persist;
	}

	private @NotNull ExternalSelectDataPersist.ExternalSelectSourcePersist buildAutoCompleteSingleData(ExternalSelectImportExport.ExternalSelectSourceImportExport data){
		ExternalSelectDataPersist.ExternalSelectSourcePersist persist = new ExternalSelectDataPersist.ExternalSelectSourcePersist();
		if (data == null) return persist;

		persist.setUrl(data.getUrl());
		persist.setOptionsRoot(data.getOptionsRoot());
		persist.setHasAuth(data.getHasAuth());
		if(data.getSourceBinding() != null) persist.setSourceBinding(this.buildOption(data.getSourceBinding()));
		if(data.getAuth() != null) persist.setAuth(this.buildAuthAutoCompleteData(data.getAuth()));
		persist.setMethod(data.getMethod());

		return persist;
	}

	private @NotNull ExternalSelectDataPersist.ExternalSelectSourceBindingPersist buildOption(ExternalSelectImportExport.ExternalSelectSourceBindingImportExport data){
		ExternalSelectDataPersist.ExternalSelectSourceBindingPersist persist = new ExternalSelectDataPersist.ExternalSelectSourceBindingPersist();
		if (data == null) return persist;

		persist.setLabel(data.getLabel());
		persist.setValue(data.getValue());
		persist.setSource(data.getSource());

		return persist;
	}

	private @NotNull ExternalSelectDataPersist.ExternalSelectAuthDataPersist buildAuthAutoCompleteData(ExternalSelectImportExport.ExternalSelectAuthImportExport data){
		ExternalSelectDataPersist.ExternalSelectAuthDataPersist persist = new ExternalSelectDataPersist.ExternalSelectAuthDataPersist();
		if (data == null) return persist;

		persist.setBody(data.getBody());
		persist.setUrl(data.getUrl());
		persist.setMethod(data.getMethod());
		persist.setPath(data.getPath());
		persist.setType(data.getType());

		return persist;
	}

	@Override
	protected ExternalSelectImportExport dataToImportExportXmlInternal(ExternalSelectDataEntity data, ExternalSelectImportExport xml) {
		xml.setMultipleSelect(data.getMultipleSelect());
		if (!this.conventionService.isListNullOrEmpty(data.getSources())){
			xml.setSources(new ArrayList<>());
			for (ExternalSelectDataEntity.ExternalSelectSourceEntity externalSelectSourceEntity : data.getSources()) {
				xml.getSources().add(this.buildAutoCompleteSingleData(externalSelectSourceEntity));
			}
		}
		return xml;
	}

	@Override
	protected boolean isMultiValueInternal(ExternalSelectDataEntity data) {
		return data.getMultipleSelect();
	}

	private @NotNull ExternalSelectImportExport.ExternalSelectSourceImportExport buildAutoCompleteSingleData(ExternalSelectDataEntity.ExternalSelectSourceEntity data){
		ExternalSelectImportExport.ExternalSelectSourceImportExport xml = new ExternalSelectImportExport.ExternalSelectSourceImportExport();
		if (data == null) return xml;

		xml.setUrl(data.getUrl());
		xml.setOptionsRoot(data.getOptionsRoot());
		xml.setHasAuth(data.getHasAuth());
		if(data.getSourceBinding() != null) xml.setSourceBinding(this.buildOption(data.getSourceBinding()));
		if(data.getAuth() != null) xml.setAuth(this.buildAuthAutoCompleteData(data.getAuth()));
		xml.setMethod(data.getMethod());

		return xml;
	}

	private @NotNull ExternalSelectImportExport.ExternalSelectSourceBindingImportExport buildOption(ExternalSelectDataEntity.ExternalSelectSourceBindingEntity data){
		ExternalSelectImportExport.ExternalSelectSourceBindingImportExport xml = new ExternalSelectImportExport.ExternalSelectSourceBindingImportExport();
		if (data == null) return xml;

		xml.setLabel(data.getLabel());
		xml.setValue(data.getValue());
		xml.setSource(data.getSource());

		return xml;
	}

	private @NotNull ExternalSelectImportExport.ExternalSelectAuthImportExport buildAuthAutoCompleteData(ExternalSelectDataEntity.ExternalSelectAuthDataEntity data){
		ExternalSelectImportExport.ExternalSelectAuthImportExport xml = new ExternalSelectImportExport.ExternalSelectAuthImportExport();
		if (data == null) return xml;

		xml.setBody(data.getBody());
		xml.setUrl(data.getUrl());
		xml.setMethod(data.getMethod());
		xml.setPath(data.getPath());
		xml.setType(data.getType());

		return xml;
	}
}
